import 'package:anagog_flutter_platform_interface/anagog_flutter_platform_interface.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';
import 'anagog_flutter_method_channel.dart';

abstract class AnagogFlutterPlatform extends PlatformInterface {
  static AnagogFlutterPlatform _instance = AnagogFlutterMethodChannel();
  static final _token = Object();
  static AnagogFlutterPlatform get instance => _instance;

  static set instance(AnagogFlutterPlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  AnagogFlutterPlatform() : super(token: _token);

  ///Sync campaigns
  ///Download all your campaigns to your app
  Future<dynamic> syncCampaigns();

  ///Simulate a campaign, use this API to test your campaigns as defined.
  ///[campaignIdentifier] A campaign identifier to simulate. Pass null to simulates all your campaigns
  Future<dynamic> simulateCampaign(String? campaignIdentifier);

  ///Create an EventStream from SDK to your Flutter app.
  ///Use this API to listen for SDK events such as onCampaignTriggered, onNotificationClick
  Stream<dynamic> obtainEventStream();

  ///Set String custom micro segment value
  ///[arguments] Micro Segment values
  Future<dynamic> setUserDefinedString(Map<String, dynamic> arguments);

  ///Get String custom micro segment
  ///[name] Stat name
  Future<String?> getUserDefinedString(String name);

  ///Set Integer custom micro segment value
  ///[arguments] Micro Segment values
  Future<dynamic> setUserDefinedInteger(Map<String, dynamic> arguments);

  ///Get Integer custom micro segment
  ///[name] Stat name
  Future<int?> getUserDefinedInteger(String name);

  ///Set Decimal custom micro segment value
  ///[arguments] Micro Segment values
  Future<dynamic> setUserDefinedDecimal(Map<String, dynamic> arguments);

  ///Get Decimal custom micro segment
  ///[name] Stat name
  Future<double?> getUserDefinedDecimal(String name);

  ///Get SDK Version
  Future<dynamic> getVersion();

  ///Get the app custom micro segments map
  Future<Map<dynamic, dynamic>?> getUserDefinedStat();

  ///Use to create report a [Micromoment] object from your app
  Future<dynamic> fireMicromoments(Micromoment micromoment);

  ///Request to generate Microsegments Snapshot report.
  Future<dynamic> requestMicrosegmentsSnapshotReport();

  ///Create and share SDK report.
  ///Use this in case you want to share with Anagog SDK operation and debugging report for analyze
  Future<dynamic> share();

  ///Generate and send SDK reports
  Future<dynamic> forceScheduleReports();

  ///Report user interaction event to SDK (e.g. Triggered, Conversion, Clicked)
  ///[campaignIdentifier] - The campaign identifier which user interact with
  ///[activityType] - Sdk activity type (e.g. clicked, conversion etc')
  Future<dynamic> setUserInteractionEvent(
      String campaignIdentifier, AnagogActivityType activityType);

  ///Get available campaigns
  Future<List<Campaign>?> getCampaigns();

  ///Get Flutter plugin version
  Future<dynamic> getPluginVersion();

  ///Returns if SDK is ready
  Future<JedAIStatus> getJedAIStatus();

  ///Enable JedAI Sdk
  Future<dynamic> enableJedAI();

  ///Disable and stop JedAI Sdk
  Future<dynamic> disableJedAI();

  ///Disable and stop JedAI. Also, all stored data by JedAI will be removed.
  Future<dynamic> disableAndPurgeJedAI();

  ///Opens JedAI SDK debugging screen
  Future<dynamic> openDebuggingScreen();
}
